import { Connection } from '@salesforce/core';
import { ComponentSet } from '../collections';
import { MetadataRequestResult, SourceApiResult, MetadataTransferResult } from './types';
export interface MetadataTransferOptions {
    usernameOrConnection: string | Connection;
    components: ComponentSet;
}
export declare abstract class MetadataTransfer<U extends MetadataRequestResult, R extends SourceApiResult | MetadataTransferResult> {
    protected components: ComponentSet;
    private signalCancel;
    private event;
    private usernameOrConnection;
    constructor({ usernameOrConnection, components }: MetadataTransferOptions);
    /**
     * Start the metadata transfer.
     *
     * @param pollInterval Frequency in milliseconds to poll for operation status
     */
    start(pollInterval?: number): Promise<R | undefined>;
    cancel(): void;
    onUpdate(subscriber: (result: U) => void): void;
    onFinish(subscriber: (result: R) => void): void;
    onCancel(subscriber: (result: U | undefined) => void): void;
    onError(subscriber: (result: Error) => void): void;
    protected getConnection(): Promise<Connection>;
    private pollStatus;
    private wait;
    protected abstract pre(): Promise<{
        id: string;
    }>;
    protected abstract checkStatus(id: string): Promise<U>;
    protected abstract post(result: U): Promise<R>;
    protected abstract doCancel(): Promise<boolean>;
}
